pub struct SFC64 { /* private fields */ }Expand description
The “Small Fast Chaotic” PRNG, originally designed by Chris Doty-Humphrey.
This PRNG holds 256 bits of state (including a 64-bit counter) and generates 64 bits
of output with each step. The minimum cycle length is $2^{64}$, and the expected
period is $~2^{255}$. Independent seeds are guaranteed to not collide within the
first $2^{64}$ steps, although the actual time to collision may be much longer.
This specific implementation passes PractRand with >2TB of output for each of several low-entropy seeds, and >1TB for sets of 8 and 64 parallel streams whose seed differs by only a single bit.
Seeding (construction)
This generator implements the [SeedableRng] trait. Any method may be used,
and the results are guaranteed to be portable.
Seeding the generator with seed_from_u64 is often the most convenient and
guarantees good pseudorandom statistics.
use hoomd_rand::SFC64;
use rand::SeedableRng;
let rng = SFC64::seed_from_u64(42);See also Seeding RNGs in the Rust Rand book.
Generation
The generators implements [TryRng] and thus also Rng.
See also the Random Values chapter in the Rust Rand book.